package eu.mhsl.craftattack.spawn.appliances.settings.datatypes;

import eu.mhsl.craftattack.spawn.appliances.settings.Settings;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.persistence.PersistentDataType;

import java.util.List;
import java.util.Objects;

public abstract class BoolSetting extends Setting<Boolean> {
    private boolean state;

    public BoolSetting(Settings.Key key) {
        super(key);
    }

    protected abstract String title();
    protected abstract String description();

    @Override
    public void fromStorage(PersistentDataContainer container) {
        this.state = container.has(getNamespacedKey())
            ? Objects.requireNonNull(container.get(getNamespacedKey(), PersistentDataType.BOOLEAN))
            : defaultValue();
    }

    @Override
    protected void toStorage(PersistentDataContainer container, Boolean value) {
        container.set(getNamespacedKey(), PersistentDataType.BOOLEAN, value);
    }

    @Override
    public ItemMeta buildMeta(ItemMeta meta) {
        meta.displayName(Component.text(title(), NamedTextColor.WHITE));
        meta.lore(List.of(
            Component.empty()
                .append(Component.text("Status: ", NamedTextColor.DARK_GRAY))
                .append(Component.text(this.state ? "An" : "Aus", this.state ? NamedTextColor.GREEN : NamedTextColor.RED)),
            Component.empty(),
            Component.text(description(), NamedTextColor.GRAY)
        ));
        return meta;
    }

    @Override
    protected void change() {
        this.state = !this.state;
    }

    @Override
    public Class<?> dataType() {
        return Boolean.class;
    }

    @Override
    public Boolean state() {
        return state;
    }
}
